package org.openmuc.jdlms.internal.lnassociation;

import org.openmuc.jasn1.ber.types.BerBitString;
import org.openmuc.jasn1.ber.types.BerInteger;
import org.openmuc.jasn1.ber.types.BerOctetString;
import org.openmuc.jdlms.internal.APdu;
import org.openmuc.jdlms.internal.AssociateSourceDiagnostic.AcseServiceProvider;
import org.openmuc.jdlms.internal.AssociateSourceDiagnostic.AcseServiceUser;
import org.openmuc.jdlms.internal.AssociationResult;
import org.openmuc.jdlms.internal.ContextId;
import org.openmuc.jdlms.internal.ObjectIdentifier;
import org.openmuc.jdlms.internal.asn1.cosem.COSEMpdu;
import org.openmuc.jdlms.internal.asn1.cosem.Conformance;
import org.openmuc.jdlms.internal.asn1.cosem.InitiateResponse;
import org.openmuc.jdlms.internal.asn1.cosem.Integer16;
import org.openmuc.jdlms.internal.asn1.cosem.Unsigned16;
import org.openmuc.jdlms.internal.asn1.cosem.Unsigned8;
import org.openmuc.jdlms.internal.asn1.iso.acse.AAREApdu;
import org.openmuc.jdlms.internal.asn1.iso.acse.ACSEApdu;
import org.openmuc.jdlms.internal.asn1.iso.acse.APTitle;
import org.openmuc.jdlms.internal.asn1.iso.acse.APTitleForm2;
import org.openmuc.jdlms.internal.asn1.iso.acse.AssociateResult;
import org.openmuc.jdlms.internal.asn1.iso.acse.AssociateSourceDiagnostic;
import org.openmuc.jdlms.internal.asn1.iso.acse.AuthenticationValue;

class InitialResponseBuilder {

    private Conformance conformanceSetting;
    private ContextId contextId;
    private AssociateResult result;
    private AssociateSourceDiagnostic associateSourceDiagnostic;
    private APTitle respondingAPTitle;
    private AuthenticationValue serverToClientChallenge;

    public InitialResponseBuilder(Conformance conformanceSetting) {
        this.conformanceSetting = conformanceSetting;

        this.contextId = ContextId.LOGICAL_NAME_REFERENCING_WITH_CIPHERING;

        this.result = new AssociateResult(AssociationResult.ACCEPTED.getValue());

        this.associateSourceDiagnostic = new AssociateSourceDiagnostic(new BerInteger(AcseServiceUser.NULL.getCode()),
                null);

        this.respondingAPTitle = null;
    }

    public InitialResponseBuilder setConformanceSetting(Conformance conformanceSetting) {
        this.conformanceSetting = conformanceSetting;
        return this;
    }

    public InitialResponseBuilder setResult(AssociationResult result) {
        this.result = new AssociateResult(result.getValue());
        return this;
    }

    public InitialResponseBuilder setSystemTitle(byte[] respondingAPTitle) {
        this.respondingAPTitle = new APTitle(new APTitleForm2(respondingAPTitle));
        return this;
    }

    public InitialResponseBuilder setAuthenticationValue(byte[] serverToClientChallenge) {
        this.serverToClientChallenge = new AuthenticationValue(new BerOctetString(serverToClientChallenge), null);
        return this;
    }

    public InitialResponseBuilder setContextId(ContextId contextId) {
        this.contextId = contextId;
        return this;
    }

    public InitialResponseBuilder setAssociateSourceDiagnostic(AcseServiceUser acseServiceUser) {
        return setAssociateSourceDiagnostic(acseServiceUser, null);
    }

    public InitialResponseBuilder setAssociateSourceDiagnostic(AcseServiceProvider acseServiceProvider) {
        return setAssociateSourceDiagnostic(null, acseServiceProvider);
    }

    private InitialResponseBuilder setAssociateSourceDiagnostic(AcseServiceUser acseServiceUser,
            AcseServiceProvider acseServiceProvider) {
        BerInteger serviceUser = null;
        BerInteger serviceProvider = null;
        if (acseServiceUser != null) {
            serviceUser = new BerInteger(acseServiceUser.getCode());
        }
        if (acseServiceProvider != null) {
            serviceProvider = new BerInteger(acseServiceProvider.getCode());
        }

        this.associateSourceDiagnostic = new AssociateSourceDiagnostic(serviceUser, serviceProvider);
        return this;
    }

    public APdu build() {

        COSEMpdu xDlmsInitiateResponsePdu = new COSEMpdu();

        InitiateResponse initiateResponse = new InitiateResponse(null, new Unsigned8(6), this.conformanceSetting,
                new Unsigned16(0xffff), new Integer16(0x0007));
        xDlmsInitiateResponsePdu.setinitiateResponse(initiateResponse);

        AAREApdu aare = new AAREApdu();
        aare.protocolVersion = new BerBitString(new byte[] { (byte) 0x80 }, 1);
        aare.applicationContextName = ObjectIdentifier.applicationContextNameFor(this.contextId);

        aare.result = this.result;

        aare.resultSourceDiagnostic = this.associateSourceDiagnostic;

        aare.respondingAPTitle = this.respondingAPTitle;

        aare.respondingAuthenticationValue = this.serverToClientChallenge;

        ACSEApdu aarqAcseAPdu = new ACSEApdu(null, aare, null, null);

        return new APdu(aarqAcseAPdu, xDlmsInitiateResponsePdu);
    }

}
