package org.openmuc.jdlms.internal;

import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.openmuc.jdlms.LnConformanceSetting;
import org.openmuc.jdlms.internal.asn1.cosem.Conformance;

public class LnConformanceConverter {

    private static final int LENGTH = 3;
    private static final int NUM_BITS = 24;

    private static final LnConformanceSetting[] VALUES = LnConformanceSetting.values();

    public static Set<LnConformanceSetting> conformanceSettingFor(Conformance conformance) {
        Set<LnConformanceSetting> settings = new HashSet<>(VALUES.length);

        byte[] bytes = conformance.value;
        int bits = (0xff & bytes[2]) | (bytes[1] << 8);
        for (LnConformanceSetting conformanceSetting : VALUES) {
            if (bitsIsSet(bits, conformanceSetting)) {
                settings.add(conformanceSetting);
            }
        }

        return settings;
    }

    private static boolean bitsIsSet(int bits, LnConformanceSetting conformanceSetting) {
        return (bits & (1 << (NUM_BITS - (1 + conformanceSetting.getIndex())))) != 0;
    }

    public static Conformance conformanceFor(LnConformanceSetting... settings) {
        return conformanceFor(Arrays.asList(settings));
    }

    public static Conformance conformanceFor(List<LnConformanceSetting> settings) {
        int bits = 0;

        for (LnConformanceSetting conformanceSetting : settings) {
            bits |= 1 << (NUM_BITS - (1 + conformanceSetting.getIndex()));
        }

        byte[] bitString = new byte[LENGTH];
        bitString[1] = (byte) ((bits & 0xFF00) >> 8);

        bitString[2] = (byte) (bits & 0xFF);

        return new Conformance(bitString, NUM_BITS);
    }

    /*
     * Don't let anyone instantiate this class.
     */
    private LnConformanceConverter() {
    }

}
